@@ -67,7 +67,7 @@
 #include "advanced-backlight.h"
 
 
-#define _(text) gettext(text)
+#define _(text) dgettext(GETTEXT_PACKAGE, text)
 
 #define NUM_PIXBUFS 7*5
 #define HILDON_STATUS_BAR_BORDER_WIDTH 20
@@ -83,6 +83,7 @@
 #define GCONF_OSSO_SOUND_DIR "/apps/osso/sound"
 #define GCONF_MASTER_VOLUME GCONF_OSSO_SOUND_DIR "/master_volume"
 
+#define VOLUME_BEEP_DELAY 300
 
 
 #define GCONF_PREFIX "/apps/adv-backlight"
@@ -106,7 +107,7 @@
 typedef struct _AdvancedBacklightPluginPrivate AdvancedBacklightPluginPrivate;
 struct _AdvancedBacklightPluginPrivate
 {
-  GdkPixbuf *pixbuf[NUM_BRIGHTNESS_ICONS + 1][NUM_VOLUME_ICONS];
+  GdkPixbuf *pixbuf[NUM_BRIGHTNESS_ICONS][NUM_VOLUME_ICONS];
   GtkWidget *icon;
   GtkWidget *button;
   GtkWidget *menu;
@@ -141,11 +142,13 @@ struct _AdvancedBacklightPluginPrivate
   gint is_770;
 
   gint notify_id;
+  guint timer_id;
   struct timeval last_beep;
   gint brightness_jump;
   gint volume_jump;
   gboolean volume_beep;
   gboolean allow_zero;
+  gint max_brightness;
 
   gint brightness_level;
   gint volume_level;
@@ -191,7 +194,6 @@ static gpointer advanced_backlight_plugin_dsme_update_brightness_thread (gpointe
 static gpointer advanced_backlight_plugin_update_volume_thread (gpointer);
 /*static void advanced_backlight_plugin_dsme_update_brightness (AdvancedBacklightPlugin *);*/
 static void advanced_backlight_plugin_configure_applet (AdvancedBacklightPlugin * vsp);
-static void advanced_backlight_plugin_do_volume_beep (AdvancedBacklightPlugin * vsp);
 static void advanced_backlight_plugin_load_icons (AdvancedBacklightPlugin * vsp);
 
 
@@ -202,6 +204,7 @@ static void advanced_backlight_plugin_update_icons (AdvancedBacklightPlugin *);
 static void advanced_backlight_plugin_load_settings (AdvancedBacklightPlugin * vsp);
 static void advanced_backlight_plugin_save_settings (AdvancedBacklightPlugin * vsp);
 static void advanced_backlight_plugin_fill_theme_list (AdvancedBacklightPlugin * vsp, GtkWidget * combobox);
+static void advanced_backlight_plugin_update_volume_fields (AdvancedBacklightPlugin * vsp);
 static int get_light_sensor (AdvancedBacklightPlugin * vsp);
 static gboolean is_n810 ();
 static gboolean is_770 ();
@@ -273,10 +276,8 @@ advanced_backlight_plugin_init (AdvancedBacklightPlugin * vsp)
   g_debug ("%s\n", __PRETTY_FUNCTION__);
 
 #ifdef ENABLE_NLS
-  setlocale (LC_ALL, "");
   bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
 #endif
   memset (priv, 0, sizeof (AdvancedBacklightPluginPrivate));
 
@@ -285,9 +286,16 @@ advanced_backlight_plugin_init (AdvancedBacklightPlugin * vsp)
   advanced_backlight_plugin_init_threads (vsp);
   advanced_backlight_plugin_init_dsme (vsp);
   advanced_backlight_plugin_check_init_rotation (vsp);
+  advanced_backlight_plugin_update_volume_fields (vsp);
   /* MERGED from ABL */
   priv->is_n810 = is_n810 ();
   priv->is_770 = is_770 ();
+
+  if (priv->is_770)
+	  priv->max_brightness = 123;
+  else
+	  priv->max_brightness = 127;
+
   gettimeofday (&priv->last_beep, NULL);
   advanced_backlight_plugin_load_settings (vsp);
 
@@ -745,7 +753,7 @@ brightness_level_changed (GtkWidget * widget, AdvancedBacklightPlugin * vsp)
 
   g_string_printf (priv->brightness_level_string, "%3d", priv->brightness_level);
   gtk_label_set_text (GTK_LABEL (priv->brightness_level_label), (priv->brightness_level_string)->str);
-  g_debug ("value=%d\n", priv->brightness_level);
+  g_debug ("priv->brightness_level=%d\n", priv->brightness_level);
   advanced_backlight_plugin_update_icons (vsp);
 
   value = g_new (int, 1);
@@ -753,13 +761,24 @@ brightness_level_changed (GtkWidget * widget, AdvancedBacklightPlugin * vsp)
   g_async_queue_push (priv->brightness_queue, value);
 }
 
+static gboolean
+volume_beep (AdvancedBacklightPlugin * vsp)
+{
+  AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
+  g_debug ("%s\n", __PRETTY_FUNCTION__);
+  if (priv->volume_beep)
+    hildon_play_system_sound (DEFAULT_BEEP);
+  priv->timer_id = 0;
+  return FALSE;                 /* One shot */
+}
+
 static void
 advanced_backlight_plugin_update_volume_value (AdvancedBacklightPlugin * vsp, gint volume)
 {
   AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
+  g_debug ("%s\n", __PRETTY_FUNCTION__);
   gconf_client_set_int (priv->gconf_client, GCONF_MASTER_VOLUME, volume, NULL);
-  if (priv->volume_beep)
-    advanced_backlight_plugin_do_volume_beep (vsp);
+  g_debug ("%s ->\n", __PRETTY_FUNCTION__);
 }
 
 static gpointer
@@ -773,6 +792,7 @@ advanced_backlight_plugin_update_volume_thread (gpointer d)
   g_async_queue_ref (priv->volume_queue);
   while ((gip = g_async_queue_pop (priv->volume_queue)) && !priv->all_done)
     {
+      g_debug ("%s [woke up]\n", __PRETTY_FUNCTION__);
       value = *(int *) gip;
       g_free (gip);
 
@@ -789,6 +809,7 @@ advanced_backlight_plugin_update_volume_thread (gpointer d)
         g_debug ("after queue pop j: %d %d\n", j, i);
       if (prev_value != value)
         {
+          g_debug ("%s setting volume\n", __PRETTY_FUNCTION__);
           advanced_backlight_plugin_update_volume_value (vsp, value);
           prev_value = value;
         }
@@ -825,7 +846,15 @@ advanced_backlight_plugin_update_volume (AdvancedBacklightPlugin * vsp)
 static void
 volume_level_changed (GtkWidget * widget, AdvancedBacklightPlugin * vsp)
 {
+  AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
   advanced_backlight_plugin_update_volume (vsp);
+  g_debug ("%s\n", __PRETTY_FUNCTION__);
+  /* We only want to beep on changes with the slider */
+  if (priv->timer_id > 0)
+    {
+      g_source_remove (priv->timer_id);
+    }
+  priv->timer_id = g_timeout_add (VOLUME_BEEP_DELAY, (GSourceFunc) volume_beep, vsp);
 }
 
 static void
@@ -854,19 +883,26 @@ get_brightness_scale (GtkWidget * widget, AdvancedBacklightPlugin * vsp)
     }
 }
 
+static void
+advanced_backlight_plugin_update_volume_fields (AdvancedBacklightPlugin * vsp)
+{
+  AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
+  gint volume = gconf_client_get_int (priv->gconf_client, GCONF_MASTER_VOLUME, NULL);
+  priv->mute = (volume < 0);
+  priv->volume_level = (priv->mute ? -1 : 1) * volume;
+}
+
 static void
 gconf_notify_volume (GConfClient * gclient, guint notify_id, GConfEntry * entry, AdvancedBacklightPlugin * vsp)
 {
   AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
-  gint volume;
 
   g_debug ("%s\n", __PRETTY_FUNCTION__);
-
-  volume = gconf_client_get_int (priv->gconf_client, GCONF_MASTER_VOLUME, NULL);
-  priv->mute = (volume < 0);
-  priv->volume_level = (priv->mute ? -1 : 1) * volume;
+  advanced_backlight_plugin_update_volume_fields (vsp);
+  g_object_freeze_notify (G_OBJECT (priv->volume_slider));
   hildon_volumebar_set_mute (HILDON_VOLUMEBAR (priv->volume_slider), priv->mute);
   hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->volume_slider), priv->volume_level);
+  g_object_thaw_notify (G_OBJECT (priv->volume_slider));
   advanced_backlight_plugin_update_icons (vsp);
 }
 
@@ -914,7 +950,7 @@ advanced_backlight_plugin_create_menu (AdvancedBacklightPlugin * vsp)
   priv->brightness_slider = hildon_hvolumebar_new ();
   gtk_container_forall (GTK_CONTAINER (priv->brightness_slider), (GtkCallback) get_brightness_scale, vsp);
   gtk_range_set_increments (GTK_RANGE (priv->brightness_scale), 1, 1);
-  gtk_range_set_range (GTK_RANGE (priv->brightness_scale), 0, 127);
+  gtk_range_set_range (GTK_RANGE (priv->brightness_scale), priv->allow_zero ? 0 : 1, priv->max_brightness);
   gtk_range_set_update_policy (GTK_RANGE (priv->brightness_scale), GTK_UPDATE_DELAYED);
   hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->brightness_slider), priv->brightness_level);
   g_object_set (G_OBJECT (priv->brightness_slider), "has_mute", FALSE, NULL);
@@ -923,6 +959,8 @@ advanced_backlight_plugin_create_menu (AdvancedBacklightPlugin * vsp)
   priv->volume_slider = hildon_hvolumebar_new ();
   gtk_container_forall (GTK_CONTAINER (priv->volume_slider), (GtkCallback) get_volume_scale, vsp);
   gtk_range_set_increments (GTK_RANGE (priv->volume_scale), 1, 1);
+  hildon_volumebar_set_mute (HILDON_VOLUMEBAR (priv->volume_slider), priv->mute);
+  hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->volume_slider), priv->volume_level);
   g_signal_connect (G_OBJECT (priv->volume_slider), "level-changed", G_CALLBACK (volume_level_changed), vsp);
   g_signal_connect (G_OBJECT (priv->volume_slider), "mute-toggled", G_CALLBACK (volume_mute_toggled), vsp);
 
@@ -955,21 +993,21 @@ advanced_backlight_plugin_create_menu (AdvancedBacklightPlugin * vsp)
   vbox = gtk_vbox_new (FALSE, 10);
 
   small_pfd = pango_font_description_copy (priv->menu->style->font_desc);
-  pango_font_description_set_size (small_pfd, (pango_font_description_get_size (small_pfd) * 3) / 4);
+  pango_font_description_set_size (small_pfd, (pango_font_description_get_size (small_pfd) * 6) / 7);
 
   hbox2 = gtk_hbox_new (TRUE, 0);
 
   label = gtk_label_new (_("Brightness"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_widget_modify_font (GTK_WIDGET (label), small_pfd);
-  gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox2), label, TRUE, TRUE, 0);
 
   priv->brightness_level_string = g_string_new ("   ");
   g_string_printf (priv->brightness_level_string, "%3d", priv->brightness_level);
   priv->brightness_level_label = gtk_label_new ((priv->brightness_level_string)->str);
-  gtk_misc_set_alignment (GTK_MISC (priv->brightness_level_label), 0.0, 0.5);
+  gtk_misc_set_alignment (GTK_MISC (priv->brightness_level_label), 1.0, 0.5);
   gtk_widget_modify_font (GTK_WIDGET (priv->brightness_level_label), small_pfd);
-  gtk_box_pack_start (GTK_BOX (hbox2), priv->brightness_level_label, TRUE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox2), priv->brightness_level_label, TRUE, TRUE, 0);
 
   gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), priv->brightness_slider, FALSE, FALSE, 0);
@@ -980,14 +1018,14 @@ advanced_backlight_plugin_create_menu (AdvancedBacklightPlugin * vsp)
   label = gtk_label_new (_("Volume"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_widget_modify_font (GTK_WIDGET (label), small_pfd);
-  gtk_box_pack_start (GTK_BOX (hbox3), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox3), label, TRUE, TRUE, 0);
 
   priv->volume_level_string = g_string_new ("   ");
   g_string_printf (priv->volume_level_string, "%4d", 50);
   priv->volume_level_label = gtk_label_new ((priv->volume_level_string)->str);
-  gtk_misc_set_alignment (GTK_MISC (priv->volume_level_label), 0.0, 0.5);
+  gtk_misc_set_alignment (GTK_MISC (priv->volume_level_label), 1.0, 0.5);
   gtk_widget_modify_font (GTK_WIDGET (priv->volume_level_label), small_pfd);
-  gtk_box_pack_start (GTK_BOX (hbox3), priv->volume_level_label, TRUE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox3), priv->volume_level_label, TRUE, TRUE, 0);
 
   gtk_box_pack_start (GTK_BOX (vbox), hbox3, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), priv->volume_slider, FALSE, FALSE, 0);
@@ -1029,7 +1067,7 @@ show_menu (GtkButton * widget, AdvancedBacklightPlugin * vsp)
   AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
 
   gboolean activate_button;
-  gint x, y, volume;
+  gint x, y;
 
   g_debug ("%s\n", __PRETTY_FUNCTION__);
 
@@ -1047,12 +1085,6 @@ show_menu (GtkButton * widget, AdvancedBacklightPlugin * vsp)
       /* Update rotation icons */
       advanced_backlight_plugin_update_rotate_button_orientation (vsp);
 
-      /* Update volume */
-      volume = gconf_client_get_int (priv->gconf_client, GCONF_MASTER_VOLUME, NULL);
-      if (volume < 0)
-        hildon_volumebar_set_mute (HILDON_VOLUMEBAR (priv->volume_slider), TRUE);
-      hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->volume_slider), abs (volume));
-
       /* Position and size */
       gdk_window_get_origin (priv->button->window, &x, &y);
       x += priv->button->allocation.x + priv->button->allocation.width - MENU_WINDOW_WIDTH;
@@ -1240,16 +1272,13 @@ advanced_backlight_plugin_dsme_update_brightness_value (AdvancedBacklightPluginP
   struct dsme_message message;
   gint conn;
 
+  g_debug ("%s\n", __PRETTY_FUNCTION__);
+  g_debug ("Setting brightness to %d\n", value);
+
   if (!priv->has_dsme)
     return;
 
-  value = (priv->brightness_level * 2);
-  if (value == 0)
-    value = 1;
-  if (value <= 2 && !priv->allow_zero)
-    value = 3;
-  if (value > 254)
-    value = 254;
+  value = (priv->brightness_level * 2) + 1;
 
   conn = dsmesock_connect ();
   message.len = 0x0c;
@@ -1312,6 +1341,8 @@ advanced_backlight_plugin_dsme_update_brightness_thread (gpointer d)
         g_debug ("after queue pop j: %d %d\n", j, i);
       if (prev_value != value)
         {
+          g_debug ("priv->brightness_level=%d\n", value);
+
           if (!is_770 ())
             advanced_backlight_plugin_dsme_update_brightness_value (priv, value);
           else
@@ -1326,7 +1357,7 @@ static void
 advanced_backlight_plugin_770_update_brightness_value (AdvancedBacklightPluginPrivate * priv, gint value)
 {
   FILE *myfile;
-  value = value * 254 + 128;    /* Math stolen directly from Fanoush's blset script. was 256 + 128 */
+  value = value * 256 + 128;    /* Math stolen directly from Fanoush's blset script. was 256 + 128 */
 
   myfile = fopen (SYSFS_BRIGHTNESS, "w");
 
@@ -1385,23 +1416,6 @@ is_770 ()
   return (version == '1');      /* 4 means n810, 3 means n800. */
 }
 
-static void
-advanced_backlight_plugin_do_volume_beep (AdvancedBacklightPlugin * vsp)
-{
-  AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
-  struct timeval current_time;
-  long time_since_beep;
-
-  gettimeofday (&current_time, NULL);
-  time_since_beep = (current_time.tv_sec - priv->last_beep.tv_sec) * 1000000 + (current_time.tv_usec - priv->last_beep.tv_usec);
-
-  if (time_since_beep > 500000)
-    {
-      hildon_play_system_sound (DEFAULT_BEEP);
-      priv->last_beep = current_time;
-    }
-}
-
 static void
 advanced_backlight_plugin_load_settings (AdvancedBacklightPlugin * vsp)
 {
@@ -1431,7 +1445,7 @@ advanced_backlight_plugin_load_settings (AdvancedBacklightPlugin * vsp)
       gconf_value_free (conf_value);
     }
   else
-    priv->allow_zero = priv->is_n810;
+    priv->allow_zero = 0;
 
   /* Selected theme */
   priv->current_theme = gconf_client_get_string (priv->gconf_client, GCONF_KEY_THEME, NULL);
@@ -1520,7 +1534,7 @@ advanced_backlight_plugin_configure_applet (AdvancedBacklightPlugin * vsp)
   size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
   dialog =
-    gtk_dialog_new_with_buttons (_("Configure Advanced Backlight"), NULL, GTK_DIALOG_MODAL,
+    gtk_dialog_new_with_buttons ("Advanced Backlight", NULL, GTK_DIALOG_MODAL,
                                  GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, NULL);
   vbox = gtk_vbox_new (FALSE, 0);
 
@@ -1540,17 +1554,17 @@ advanced_backlight_plugin_configure_applet (AdvancedBacklightPlugin * vsp)
   choose_theme = gtk_combo_box_new_text ();
   advanced_backlight_plugin_fill_theme_list (vsp, choose_theme);
 
-  theme_caption = hildon_caption_new (NULL, _("Choose theme"), choose_theme, NULL, HILDON_CAPTION_MANDATORY);
+  theme_caption = hildon_caption_new (NULL, _("Theme"), choose_theme, NULL, HILDON_CAPTION_MANDATORY);
   hildon_caption_set_size_group (HILDON_CAPTION (theme_caption), GTK_SIZE_GROUP (size_group));
 
   brightness_spinner = hildon_number_editor_new (1, 255);
   hildon_number_editor_set_value (HILDON_NUMBER_EDITOR (brightness_spinner), priv->brightness_jump);
-  brightness_caption = hildon_caption_new (NULL, _("Brightness interval"), brightness_spinner, NULL, HILDON_CAPTION_MANDATORY);
+  brightness_caption = hildon_caption_new (NULL, _("Brightness step size"), brightness_spinner, NULL, HILDON_CAPTION_MANDATORY);
   hildon_caption_set_size_group (HILDON_CAPTION (brightness_caption), GTK_SIZE_GROUP (size_group));
 
   volume_spinner = hildon_number_editor_new (1, 100);
   hildon_number_editor_set_value (HILDON_NUMBER_EDITOR (volume_spinner), priv->volume_jump);
-  volume_caption = hildon_caption_new (NULL, _("Volume interval"), volume_spinner, NULL, HILDON_CAPTION_MANDATORY);
+  volume_caption = hildon_caption_new (NULL, _("Volume step size"), volume_spinner, NULL, HILDON_CAPTION_MANDATORY);
   hildon_caption_set_size_group (HILDON_CAPTION (volume_caption), GTK_SIZE_GROUP (size_group));
 
   gtk_box_pack_start (GTK_BOX (vbox), theme_caption, FALSE, FALSE, 0);
@@ -1588,6 +1602,13 @@ advanced_backlight_plugin_configure_applet (AdvancedBacklightPlugin * vsp)
           gtk_widget_destroy (confirmation_dialog);
         }
 
+      gtk_range_set_range (GTK_RANGE (priv->brightness_scale), priv->allow_zero ? 0 : 1, priv->max_brightness);
+      if (!priv->allow_zero && priv->brightness_level == 0)
+        {
+          /* If allow zero was disabled, ensure that the actual value is greater than zero */
+          priv->brightness_level = 1;
+          hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->brightness_slider), priv->brightness_level);
+        }
 
       priv->brightness_jump = hildon_number_editor_get_value (HILDON_NUMBER_EDITOR (brightness_spinner));
       priv->volume_jump = hildon_number_editor_get_value (HILDON_NUMBER_EDITOR (volume_spinner));
@@ -1643,14 +1664,14 @@ advanced_backlight_plugin_load_icons (AdvancedBacklightPlugin * vsp)
   gchar *file_name;
   GError *err;
 
-  for (i = 1; i <= NUM_BRIGHTNESS_ICONS; i++)
+  for (i = 0; i < NUM_BRIGHTNESS_ICONS; i++)
     {
       for (j = 0; j < NUM_VOLUME_ICONS; j++)
         {
           if (j == NUM_VOLUME_ICONS - 1)
-            snprintf (icon_name, 30, "adv-backlight-icon-%d.m.png", i);
+            snprintf (icon_name, 30, "adv-backlight-icon-%d.m.png", i + 1);
           else
-            snprintf (icon_name, 30, "adv-backlight-icon-%d.%d.png", i, j);
+            snprintf (icon_name, 30, "adv-backlight-icon-%d.%d.png", i + 1, j);
 
           file_name = g_build_filename (ICON_DIR, priv->current_theme, icon_name, NULL);
           if (priv->pixbuf[i][j] != NULL)
@@ -1687,9 +1708,9 @@ advanced_backlight_plugin_update_icons (AdvancedBacklightPlugin * vsp)
         volume_icon = NUM_VOLUME_ICONS - 2;
     }
 
-  brightness_icon = ((NUM_BRIGHTNESS_ICONS - 1) * priv->brightness_level) / 100 + 1;
-  if (brightness_icon > NUM_BRIGHTNESS_ICONS)
-    brightness_icon = NUM_BRIGHTNESS_ICONS;
+  brightness_icon = ((NUM_BRIGHTNESS_ICONS - 1) * priv->brightness_level) / priv->max_brightness;
+  if (brightness_icon > NUM_BRIGHTNESS_ICONS - 1)
+    brightness_icon = NUM_BRIGHTNESS_ICONS - 1;
 
   if (!priv->icon)
     priv->icon = gtk_image_new_from_pixbuf (priv->pixbuf[brightness_icon][volume_icon]);
